package com.diagbot.dto;

import com.diagbot.entity.QuestionDetail;
import com.diagbot.entity.QuestionMapping;
import com.diagbot.enums.ControlTypeEnum;
import com.diagbot.enums.QuestionTypeEnum;
import com.diagbot.enums.TagTypeEnum;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 标签基础表
 * </p>
 *
 * @author zhoutg
 * @since 2018-12-03
 */

@Getter
@Setter
public class QuestionPageDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    private Long id;

    /**
     * 是否删除,N:未删除，Y:删除
     */
    private String isDeleted;

    /**
     * 记录创建时间
     */
    private Date gmtCreate;

    /**
     * 记录修改时间,如果时间是1970年则表示纪录未修改
     */
    private Date gmtModified;

    /**
     * 创建人，0表示无创建人值
     */
    private String creator;

    /**
     * 修改人,如果为0则表示纪录未修改
     */
    private String modifier;

    /**
     * 标签名
     */
    private String tagName;

    /**
     * 内容
     */
    private String name;

    /**
     * 性别(1：男 2：女 3：通用)
     */
    private Integer sexType;

    /**
     * 最小年龄
     */
    private Integer ageBegin;

    /**
     * 最大年龄
     */
    private Integer ageEnd;

    /**
     * 标签标识(1：单项单列，例如： 程度，咳出通畅度  2：单项多列，例如：杂音，添加症状 3：横铺单标签形式，例如：血压 4：横铺多标签形式，例如：咳嗽 5：竖铺组合项，例如：有无治疗 6：组合项多列 既往史 7：化验)
     */
    private Integer tagType;

    /**
     * 标签标识（中文）
     */
    private String tagTypeCn;

    /**
     * 控件类型(0：无类型，默认值 1：下拉单选 2：下拉多选 3：纯文本 4：待定 5：待定 6：文本框 7：数字键盘文本框 99：联合推送)
     */
    private Integer controlType;
    /**
     * 控件类型（中文）
     */
    private String controlTypeCn;

    /**
     * 类型
     */
    private Integer type;

    /**
     * 类型(中文名称)
     */
    private String typeCn;

    /**
     * 检索类型
     */
    private Integer subType;

    /**
     * 元素类型
     */
    private Integer itemType;

    /**
     * 前置内容
     */
    private String labelPrefix;

    /**
     * 后置内容
     */
    private String labelSuffix;

    /**
     * 最小值
     */
    private BigDecimal minValue;

    /**
     * 最大值
     */
    private BigDecimal maxValue;

    /**
     * 文本生成规则(0：点选顺序，例如初为 1：从上到下，从左到右，例如查体杂音 2：从左到右，从上到下，例如伴/无)
     */
    private Integer textGenerate;

    /**
     * 是否复制 (0：不复制，1：复制)
     */
    private Integer copyType;

    /**
     * 复制文字
     */
    private String showAdd;

    private String formulaCode; //公式编码

    /**
     * 备注
     */
    private String remark;

    /**
     * 明细
     */
    private List<QuestionDetail> questionDetails = new ArrayList<>();

    /**
     * 映射表
     */
    private List<QuestionMapping> questionMappings = new ArrayList<>();



    /*****************************将类型转换成中文返回*****************************************/
    public String getTypeCn() {
        return QuestionTypeEnum.getName(this.type);
    }
    public String getTagTypeCn() {
        return this.tagType == null ? "" : TagTypeEnum.getName(this.tagType);
    }
    public String getControlTypeCn() {
        return this.controlType == null ? "" : ControlTypeEnum.getName(this.controlType);
    }
    /**********************************************************************/
}
