package com.diagbot.facade;

import com.diagbot.dto.ConceptDTO;
import com.diagbot.entity.LibraryInfo;
import java.util.List;
import com.diagbot.service.impl.ConceptServiceImpl;
import com.diagbot.vo.TermVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @Description: 术语查询业务层
 * @author: Weixuan Huang
 * @time: 2019/1/14 16:17
 */
@Component
public class ConceptFacade extends ConceptServiceImpl {

    @Autowired
    private LibraryInfoFacade libraryinfoFacade;
    @Autowired
    private LibraryDetailFacade libraryDetailFacade;
    @Autowired
    private MedicalFacade medicalFacade;


    /**
     * 获取医学术语信息
     *
     * @param termvo
     * @return TermDTO
     */
    public ConceptDTO getConceptInfo(@RequestBody TermVo termvo) {

        ConceptDTO conceptDTO = getConcept(termvo);
        // 获取与医学术语的基本信息
        termvo.setId(conceptDTO.getLibId());
        termvo.setConcept_id(conceptDTO.getId());
        List<LibraryInfo> libraryInfo = libraryinfoFacade.getLibraryInfor(termvo);
        for (LibraryInfo libinfo : libraryInfo) {
            if (libinfo.getIsConcept() != null && libinfo.getIsConcept() == 1) {
                conceptDTO.setName(libinfo.getName());
            }
        }
        conceptDTO.setLibraryInfo(libraryInfo);

        // 获取与术语相关的医学知识
        conceptDTO.setInformation(libraryDetailFacade.getLibraryDetails(termvo));
        // 获取与术语相关的临床医学信息
        conceptDTO.setMedicalInfo(medicalFacade.getMedicalInfo(termvo));
//        // 获取术语的所有上级信息
//        List<LevelDTO> levelDTOList = levelFacade.getInfor(libraryDTO);
//        for (LevelDTO item : levelDTOList) {
//            while (item.getUplevel() != null) {
//                item.setUplevel_val(getOne("id", item.getUplevel().toString()).getName());
//                item.setGrpId_val(getOne("id", item.getGrpId().toString()).getName());
//            }
//        }
//        termDTO.setLevelInfo(levelDTOList);

        return conceptDTO;
    }

}
