package com.diagbot.web;


import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.LibraryDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.TermDTO;
import com.diagbot.facade.LibraryFacade;
import com.diagbot.vo.AddTermVo;
import com.diagbot.vo.TermVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author Weixuan Huang
 * @since 2019-01-14
 */
@RestController
@RequestMapping("/library")
@Api(value = "医学术语API", tags = { "医学术语API" })

public class LibraryController {
    @Autowired
    private LibraryFacade libraryFacade;

    @ApiOperation(value = "术语信息查询",
            notes = "Term：术语名称，必填<br>")
    @PostMapping("/getTermInfo")
    @SysLogger("getTermInfo")
    public RespDTO<TermDTO> getTermInfo(@Valid @RequestBody TermVo termvo) {
        TermDTO data = libraryFacade.getTermInfo(termvo);

        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "术语基本信息查询",
            notes = "Term: 术语名称，必填<br>")
    @PostMapping("/getTerm")
    @SysLogger("getTerm")
    public RespDTO<LibraryDTO> getTerm(@Valid @RequestBody TermVo termVo) {
        LibraryDTO libraryDTO = libraryFacade.getTerm(termVo);

        return RespDTO.onSuc(libraryDTO);
    }

    @ApiOperation(value = "增添术语信息", notes = "Term: 术语名称，必填<br>")
    @PostMapping("/addTerm")
    @SysLogger("addTerm")
    @Transactional
    public RespDTO<Boolean> addTerm(AddTermVo addTermVo) {

        Boolean result = libraryFacade.addTerm(addTermVo);
        return RespDTO.onSuc(result);
    }
}
